def generate_setup_script(snapshot):
    lines = ["#!/bin/bash", "set -e"]
    os_info = snapshot['os']
    packages = snapshot['packages']
    editors = snapshot['editors']
    dbs = snapshot['databases']
    env_vars = snapshot.get('env_vars', {})
    shell_configs = snapshot.get('shell_configs', {})
    lines.append("sudo apt-get update")
    apt_packages = editors + dbs
    if packages['apt']:
        for line in packages['apt']:
            if '/' in line:
                pkg = line.split('/')[0]
                if pkg not in apt_packages:
                    apt_packages.append(pkg)
    if apt_packages:
        lines.append(f"sudo apt-get install -y {' '.join(apt_packages)}")
    lines.append("sudo apt-get install -y python3-pip")
    if packages['pip']:
        lines.append(f"pip3 install {' '.join([pkg.split('==')[0] for pkg in packages['pip']])}")
    if packages.get('npm'):
        npm_pkgs = list(packages['npm'].keys())
        if npm_pkgs:
            lines.append(f"npm install -g {' '.join(npm_pkgs)}")
    for k, v in env_vars.items():
        lines.append(f'export {k}="{v}"')
    for fname, content in shell_configs.items():
        lines.append(f"{fname}.bak")
        lines.append(content)
    return '\n'.join(lines)



import platform
import subprocess
import json
import os

def get_os_info():
    return {
        'system': platform.system(),
        'release': platform.release(),
        'version': platform.version(),
        'machine': platform.machine(),}
def get_env_vars():
    blacklist = {'PWD', 'OLDPWD', 'LS_COLORS', 'SSH_AUTH_SOCK', 'XDG_SESSION_ID', 'XDG_RUNTIME_DIR', 'HOME', 'USER', 'PATH', 'SHELL'}
    return {k: v for k, v in os.environ.items() if k not in blacklist}
def get_shell_configs():
    home = os.path.expanduser('~')
    configs = {}
    for fname in ['.bashrc', '.zshrc', '.profile', '.bash_profile']:
        path = os.path.join(home, fname)
        if os.path.exists(path):
            try:
                with open(path, 'r') as f:
                    configs[fname] = f.read()
            except Exception:
                configs[fname] = '<unreadable>'
    return configs
def get_npm_packages():
    try:
        npm = subprocess.getoutput('npm list -g --depth=0 --json')
        return json.loads(npm).get('dependencies', {})
    except Exception:
        return {}
def get_installed_packages():
    pip = subprocess.getoutput('pip freeze')
    try:
        apt = subprocess.getoutput('apt-mark showmanual')
    except Exception:
        apt = ''
    return {
        'pip': pip.splitlines(),
        'apt': apt.splitlines(),
        'npm': get_npm_packages(),
    }
def get_editors():
    editors = []
    for editor in ['code', 'vim', 'nano', 'emacs', 'subl']:
        if subprocess.getoutput(f'which {editor}'):
            editors.append(editor)
    return editors
def get_databases():
    dbs = []
    for db in ['mysql', 'psql', 'mongod', 'redis-server']:
        if subprocess.getoutput(f'which {db}'):
            dbs.append(db)
    return dbs
def get_open_ports():
    try:
        output = subprocess.getoutput("ss -tuln")
        return output.splitlines()
    except Exception:
        return []
def snapshot_system():
    snapshot = {
        'os': get_os_info(),
        'env_vars': get_env_vars(),
        'shell_configs': get_shell_configs(),
        'packages': get_installed_packages(),
        'editors': get_editors(),
        'databases': get_databases(),
        'ports': get_open_ports(),
    }
    return snapshot



def generate_dockerfile(snapshot):
    os_info = snapshot['os']
    packages = snapshot['packages']
    editors = snapshot['editors']
    dbs = snapshot['databases']
    base = 'ubuntu:latest' if os_info['system'].lower() == 'linux' else 'debian:latest'
    dockerfile = [f'FROM {base}', 'RUN apt-get update && apt-get install -y \\']
    apt_packages = editors + dbs
    if packages['apt']:
        for line in packages['apt']:
            if '/' in line:
                pkg = line.split('/')[0]
                if pkg not in apt_packages:
                    apt_packages.append(pkg)
    dockerfile.append('    ' + ' \\\n    '.join(apt_packages))
    dockerfile.append('RUN apt-get clean')
    dockerfile.append('RUN apt-get install -y python3-pip')
    if packages['pip']:
        dockerfile.append('RUN pip3 install ' + ' '.join([pkg.split('==')[0] for pkg in packages['pip']]))
    dockerfile.append('CMD ["/bin/bash"]')
    return '\n'.join(dockerfile)



import argparse
from .snapshot import snapshot_system
from .dockerize import generate_dockerfile
from .setupscript import generate_setup_script
from .share import export_snapshot

def main():
    parser = argparse.ArgumentParser(description="EnvTwin: Instant Environment Recreator")
    parser.add_argument('--dockerfile', action='store_true', help='Generate Dockerfile from current environment or if there is no enviroment it will list all packages of the linux OS ur using (Windows not tested yet)')
    parser.add_argument('--setup-script', action='store_true', help='Generate setup script from current environment')
    parser.add_argument('--export', type=str, help='Export snapshot to file')
    args = parser.parse_args()

    snapshot = snapshot_system()
    if args.dockerfile:
        dockerfile = generate_dockerfile(snapshot)
        with open('Dockerfile', 'w') as f:
            f.write(dockerfile)
        print('Dockerfile written to Dockerfile')
    if args.setup_script:
        setup_script = generate_setup_script(snapshot)
        with open('setup.sh', 'w') as f:
            f.write(setup_script)
        print('Setup script written to setup.sh')
    if args.export:
        export_snapshot(snapshot, args.export)

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="envtwin",
    version="1.1.0",
    description="Instant Environment Recreator: snapshot and share your dev environment.",
    author="Syntax-XXX",
    packages=find_packages(),
    install_requires=[],
    entry_points={
        "console_scripts": [
            "envtwin=envtwin.__main__:main"
        ]
    },
    python_requires=">=3.6",
    include_package_data=True,
    license="MIT",
)
